package com.persagy.dmp.starter.alarm.communication.netty;

import com.persagy.dmp.starter.alarm.service.NettyAlarmService;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioChannelOption;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import io.netty.handler.codec.LengthFieldPrepender;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;
import io.netty.handler.timeout.IdleStateHandler;
import io.netty.util.concurrent.DefaultThreadFactory;
import io.netty.util.concurrent.UnorderedThreadPoolEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;
import java.util.concurrent.TimeUnit;

/**
 * @description: netty报警服务端
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/30 10:54 上午
 * @version: V1.0
 */
@Component
@Slf4j
public class NettyAlarmServer {
    @Autowired
    private NettyAlarmMsgBaseHandler nettyAlarmMsgBaseHandler;

    @Value("${group.alarm.port}")
    public int port;

    /**
     * 用于接收客户端的TCP连接
     */
    EventLoopGroup bossGroup = null;
    /**
     * 处理I/O相关的读写操作，或者执行系统Task、定时任务Task等。
     */
    EventLoopGroup workGroup = null;
    ChannelFuture channelFuture = null;

    /**
     * @Title: start
     * @Description: 启动netty服务端
     */
    public void start() {
        bossGroup = new NioEventLoopGroup();
        workGroup = new NioEventLoopGroup();
        UnorderedThreadPoolEventExecutor businessGroup = new UnorderedThreadPoolEventExecutor(10, new DefaultThreadFactory("business"));

        try {
            // （服务端启动类）ServerBootstrap负责初始化netty服务器，并且开始监听端口的socket请求
            ServerBootstrap startNetty = new ServerBootstrap();
            startNetty.group(bossGroup, workGroup)
                    .channel(NioServerSocketChannel.class)
                    .option(NioChannelOption.SO_BACKLOG, 1024)
                    .childOption(NioChannelOption.TCP_NODELAY, true)
                    .handler(new LoggingHandler(LogLevel.INFO))
                    .childHandler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel ch) throws Exception {
                            // 解决粘包和拆包问题，增加的编码和解码器
                            ch.pipeline().addLast(new LengthFieldBasedFrameDecoder(Integer.MAX_VALUE, 0, 4, 0, 4));
                            ch.pipeline().addLast(new LengthFieldPrepender(4));
                            // the encoder and decoder are static as these are sharable
                            ch.pipeline().addLast(new StringDecoder());
                            ch.pipeline().addLast(new StringEncoder());
                            // 为监听客户端read/write事件的Channel添加用户自定义的ChannelHandler
                            ch.pipeline().addLast(businessGroup, nettyAlarmMsgBaseHandler);

                            // 增加超时检查
                            ch.pipeline().addLast(new IdleStateHandler(10, 0, 0, TimeUnit.SECONDS));
                        }
                    });
            channelFuture = startNetty.bind(new InetSocketAddress(port)).sync();

            System.out.println("------服务端启动closeFuture前------------");
            channelFuture.channel().closeFuture().sync();
            System.out.println("------服务端启动closeFuture后------------");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // if (channelFuture != null && channelFuture.isSuccess()) {
            // System.out.println("------netty启动成功------------");
            // } else {
            // System.out.println("------netty启动失败------------");
            // }
            // 关闭主线程组
            bossGroup.shutdownGracefully();
            // 关闭工作线程组
            workGroup.shutdownGracefully();
            businessGroup.shutdownGracefully();

        }
    }


}
