package com.persagy.dmp.starter.alarm.communication.websocket;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @description: websocket服务端
 * 由于是websocket 所以原本是@RestController的http形式直接替换成@ServerEndpoint即可，作用是一样的 就是指定一个地址表示定义一个websocket的Server端
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/30 6:47 下午
 * @version: V1.0
 */
@ServerEndpoint(value = "/websocket/iot")
@Component
@Slf4j
public class AlarmWebSocketServer {

    @OnOpen
    public void onOpen(Session session) {
        log.info("有新的客户端建立连接，编号: " + session.getId());
        //将新用户存入在线的组
        String projectIdStr = getProjectIdStr(session);
        AlarmWebSocketCache.addClient(projectIdStr, session);
    }

    /**
     * @description: 获取查询条件中的projectId
     * @param: session
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/2 6:53 下午
     * @version: V1.0
     */
    private String getProjectIdStr(Session session) {
        Map<String, List<String>> requestParameterMap = session.getRequestParameterMap();
        if (requestParameterMap == null) {
            return null;
        }
        List<String> projectIds = requestParameterMap.get("projectId");
        if (CollectionUtils.isEmpty(projectIds)) {
            return null;
        }
        return projectIds.get(0);
    }

    /**
     * 客户端关闭
     *
     * @param session session
     */
    @OnClose
    public void onClose(Session session) {
        String clientId = session.getId();
        log.info("客户端断开连接，编号:" + clientId);
        //将掉线的用户移除在线的组里
        String projectIdStr = getProjectIdStr(session);
        AlarmWebSocketCache.removeClient(projectIdStr, clientId);
    }

    /**
     * 发生错误
     *
     * @param throwable e
     */
    @OnError
    public void onError(Session session, Throwable throwable) {
        String id = "";
        if (null != session) {
            id = session.getId();
            String projectIdStr = getProjectIdStr(session);
            AlarmWebSocketCache.removeClient(projectIdStr, id);
        }
        log.info("客户端{}出错 ", id);
    }

    /**
     * 收到客户端发来消息
     *
     * @param message 消息对象，格式：point,org,1101010001,10001-101,10001-102,10001-103,10001-104
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        String clientId = session.getId();
        log.info("收到客户端{}的数据请求消息:{}", clientId, message);
        if ("closeSession".equals(message)) {
            onClose(session);
            return;
        }
    }


    /**
     * @param projectId 项目ID
     * @param msg       消息
     * @description: 发送消息
     * @return: void
     * @exception:
     * @author: shiliqiang
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 22:30
     * @version: V1.0
     */
    public static void sendMsgToClients(String projectId, String msg) throws Exception {
        Set<String> projectSessionIds = AlarmWebSocketCache.getProjectSessionIds(projectId);
        if (!CollectionUtils.isEmpty(projectSessionIds)) {
            for (String sessionId : projectSessionIds) {
                Session session = AlarmWebSocketCache.getClient(sessionId);
                if (null != session && session.isOpen()) {
                    synchronized (session) {
                        //同步发送
                        session.getBasicRemote().sendText(msg);
                    }
                }
            }
        }
    }
}
